@@ -202,7 +202,7 @@
 #define PRI_LAYER_1_ALAW			0x23
 #define PRI_LAYER_1_G721			0x24
 #define PRI_LAYER_1_G722_G725		0x25
-#define PRI_LAYER_1_G7XX_384K		0x26
+#define PRI_LAYER_1_H223_H245		0x26
 #define PRI_LAYER_1_NON_ITU_ADAPT	0x27
 #define PRI_LAYER_1_V120_RATE_ADAPT	0x28
 #define PRI_LAYER_1_X31_RATE_ADAPT	0x29
@@ -386,7 +386,7 @@ typedef union {
 	pri_event_ring	  ring;		/* Ring */
 	pri_event_hangup  hangup;	/* Hang up */
 	pri_event_ringing ringing;	/* Ringing */
-	pri_event_ringing answer;	/* Answer */
+	pri_event_answer  answer;	/* Answer */
 	pri_event_restart_ack restartack;	/* Restart Acknowledge */
 	pri_event_proceeding  proceeding;	/* Call proceeding & Progress */
 	pri_event_setup_ack   setup_ack;	/* SETUP_ACKNOWLEDGE structure */
