@@ -42,11 +42,7 @@
 #include <sys/time.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
-#if defined(__linux__)
-#include <linux/zaptel.h>
-#elif defined(__FreeBSD__) || defined(SOLARIS)
-#include <zaptel.h>
-#endif
+#include <zaptel/zaptel.h>
 #ifndef SOLARIS
 #include <zap.h>
 #endif
@@ -147,7 +143,7 @@ static void event2(struct pri *pri, pri_event *e)
 	}
 }
 
-static void testmsg(char *s)
+static void testmsg(struct pri *pri, char *s)
 {
 	char *c;
 	static int keeplast = 0;
@@ -173,7 +169,7 @@ static void testmsg(char *s)
 		keeplast = 0;
 }
 
-static void testerr(char *s)
+static void testerr(struct pri *pri, char *s)
 {
 	char *c;
 	static int keeplast = 0;
@@ -264,6 +260,7 @@ int main(int argc, char *argv[])
 	}
 	first = pri;
 	pri_set_debug(pri, DEBUG_LEVEL);
+	pri_facility_enable(pri);
 	if (pthread_create(&tmp, NULL, dchan, pri)) {
 		perror("thread(0)");
 		exit(1);
@@ -273,6 +270,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 	pri_set_debug(pri, DEBUG_LEVEL);
+	pri_facility_enable(pri);
 	if (pthread_create(&tmp, NULL, dchan, pri)) {
 		perror("thread(1)");
 		exit(1);
